///<reference path="node_modules/typescript/bin/typescript.d.ts" />
///<reference path="node_modules/typescript/bin/typescript_internal.d.ts" />
///<reference path="node_modules/typescript/bin/typescriptServices.d.ts" />
///<reference path="typings/node/node.d.ts" />
///<reference path="typings/q/Q.d.ts" />
var typescript = require('typescript');
var path = require('path');
var fs = require('fs');
var Q = require('q');
var readFile = Q.denodeify(fs.readFile);
var compilerOptions = {
    target: 1 /* ES5 */,
    module: 1 /* CommonJS */,
    sourceMap: true
};
var files = {};
var libPath = path.join(path.dirname(require.resolve('typescript')), 'lib.d.ts');
files[libPath] = {
    text: fs.readFileSync(libPath, 'utf-8'),
    version: 0
};
var servicesHost = {
    getScriptFileNames: function () { return Object.keys(files); },
    getScriptVersion: function (filename) { return files[filename] && files[filename].version.toString(); },
    getScriptSnapshot: function (filename) {
        var file = files[filename];
        return {
            getText: function (start, end) { return file.text.substring(start, end); },
            getLength: function () { return file.text.length; },
            getLineStartPositions: function () { return []; },
            getChangeRange: function (oldSnapshot) { return undefined; }
        };
    },
    getCurrentDirectory: function () { return process.cwd(); },
    getScriptIsOpen: function () { return true; },
    getCompilationSettings: function () { return compilerOptions; },
    getDefaultLibFilename: function (options) { return 'lib.d.ts'; },
    log: function (message) { return console.log(message); }
};
var langService = typescript.createLanguageService(servicesHost, typescript.createDocumentRegistry());
function rootReferencePath(referencePath, dirname) {
    return typescript.isRootedDiskPath(referencePath) ? referencePath : typescript.combinePaths(dirname, referencePath);
}
function loadDependenciesAndTransform(filePath, contents) {
    var _this = this;
    filePath = path.normalize(filePath);
    var resolve = Q.denodeify(this.resolve.bind(this));
    if (!Object.prototype.hasOwnProperty.call(files, filePath)) {
        var fileInfo = typescript.preProcessFile(contents);
        var dirname = path.dirname(filePath);
        var dependencies = fileInfo.referencedFiles.concat(fileInfo.importedFiles).map(function (f) { return ({
            original: f.filename,
            resolved: '',
            pos: f.pos,
            end: f.end,
            reference: fileInfo.referencedFiles.indexOf(f) != -1
        }); });
        files[filePath] = { version: 0, text: contents };
        return Q.all(dependencies.map(function (f) { return resolve(dirname, f.reference ? rootReferencePath(f.original, dirname) : f.original).then(function (newPath) { return f.resolved = newPath; }).then(function (newPath) { return f; }); })).then(function (filePaths) { return filePaths.filter(function (f) { return f.resolved.match(/\.ts$/) != null; }); }).then(function (filePaths) { return Q.all(filePaths.map(function (f) { return readFile(f.resolved, 'utf-8').then(function (fileContents) { return loadDependenciesAndTransform.call(_this, f.resolved, fileContents); }); })); }).then(function () { return contents; });
    }
    return Q(contents);
}
function loader(contents) {
    var _this = this;
    this.cacheable && this.cacheable();
    var callback = this.async();
    var filePath = path.normalize(this.resourcePath);
    Q.fcall(loadDependenciesAndTransform.bind(this), filePath, contents).then(function (contents) {
        var file = files[filePath];
        file.text = contents;
        file.version++;
        var output = langService.getEmitOutput(filePath);
        var diagnostics = langService.getCompilerOptionsDiagnostics().concat(langService.getSyntacticDiagnostics(filePath)).concat(langService.getSemanticDiagnostics(filePath)).forEach(function (diagnostic) {
            if (diagnostic.file) {
                var lineChar = diagnostic.file.getLineAndCharacterFromPosition(diagnostic.start);
                _this.emitError("  " + (diagnostic.file && diagnostic.file.filename) + " (" + lineChar.line + "," + lineChar.character + "): " + diagnostic.messageText);
            }
            else {
                _this.emitError("  unknown file: " + diagnostic.messageText);
            }
        });
        if (output.outputFiles.length == 0)
            throw new Error("Typescript emitted no output for " + filePath);
        contents = output.outputFiles[1].text;
        var sourceMap = JSON.parse(output.outputFiles[0].text);
        sourceMap.sourcesContent = [contents];
        return [contents, sourceMap];
    }).done(function (contents) { return callback(null, contents[0], contents[1]); }, function (err) { return callback(err); });
}
module.exports = loader;
